////////////////////////////////////////
//
// Free for private and commercial use.
//
////////////////////////////////////////

#include <zneo.h>

#include "zneo2.h"
#include "hyper.h"

#define PAGE_SIZE	2048ul

static void EraseFlash(uint32_t uAddr);
static void UnlockFlash(uint32_t uAddr);
static void LockFlash(void);

// This code section is used only for Boot Loader update and is located at the beginning of RAM

// Copy boot loader backup from RAM to Flash and execute updated program
void HyperFlash(uint32_t uDstAddr, uint32_t uSrcAddr, uint32_t uSize)
{
	int16_t sPage = -1;

	while (uSize)
	{
		if (uDstAddr / PAGE_SIZE != sPage)
		{	// new page
			sPage = uDstAddr / PAGE_SIZE;
			EraseFlash(uDstAddr);		// erase page
			UnlockFlash(uDstAddr);
		}

		if (!(uDstAddr & 1))
			*(uint8_t erom*)uDstAddr++ = (--uSize, *(uint8_t erom*)uSrcAddr++);
		else	// the copy process begins on a odd address
			*(uint8_t erom*)(uDstAddr - 1) = 0xFF;

		if (uSize)
			*(uint8_t erom*)uDstAddr++ = (--uSize, *(uint8_t erom*)uSrcAddr++);
		else	// the copy process ends on a even address
			*(uint8_t erom*)uDstAddr = 0xFF;
	}

	LockFlash();

	// Executing updated boot loader
	((void(*)(void))*(uint32_t*)RESET)();
}

// Lock the flash
static void LockFlash(void)
{
	while ((FSTAT & FSTAT_FSTAT) != FSTAT_FSTAT_IDLE) ;	// Flash Controller idle?

	while (FSTAT & FSTAT_UNLOCK)
		FCMD = 0;	// write some value other than 0x73, 0x8c, 0x95, and 0x63
}

// Unlock the flash page in which addr resides
static void UnlockFlash(uint32_t uAddr)
{
	LockFlash();

	FPAGE = (uAddr >> 8) & FPAGE_PAGE;
	FCMD = FCMD_UNLOCK_FIRST;	// First unlock command
	FCMD = FCMD_UNLOCK_SECOND; 	// Second unlock command
}

// Erase the flash page in which addr resides
static void EraseFlash(uint32_t uAddr)
{
	UnlockFlash(uAddr);

	// Page erase command
	for (FCMD = FCMD_PAGE_ERASE; (FSTAT & FSTAT_FSTAT) != FSTAT_FSTAT_IDLE; ) ;
}
