////////////////////////////////////////
//
// Free for private and commercial use.
//
////////////////////////////////////////

#include <zneo.h>
#include "zneo2.h"

#include "flash.h"

// Initialize flash frequency register
void InitFlash(uint32_t uFreq)
{
	FFREQ = (uint16_t)(uFreq / 1000);
}

// Lock the flash
void LockFlash(void)
{
	while ((FSTAT & FSTAT_FSTAT) != FSTAT_FSTAT_IDLE) ;	// Flash Controller idle?

	while (FSTAT & FSTAT_UNLOCK)
		FCMD = 0;	// write some value other than 0x73, 0x8c, 0x95, and 0x63
}

// Unlock the flash page in which addr resides
void UnlockFlash(uint32_t uAddr)
{
	LockFlash();

	FPAGE = (uAddr >> 8) & FPAGE_PAGE;
	FCMD = FCMD_UNLOCK_FIRST;	// First unlock command
	FCMD = FCMD_UNLOCK_SECOND; 	// Second unlock command
}

// Erase the flash page in which addr resides
void EraseFlash(uint32_t uAddr) 
{
	UnlockFlash(uAddr);

	// Page erase command
	for (FCMD = FCMD_PAGE_ERASE; (FSTAT & FSTAT_FSTAT) != FSTAT_FSTAT_IDLE; ) ;
}

// This function writes a single byte value to the erased Flash location.
BOOL WriteFlash(uint32_t uAddr, uint8_t ucValue)
{
	uint16_t usValue;

	if (*(uint8_t erom*)uAddr != ucValue && (*(uint8_t erom*)uAddr & ucValue) == ucValue)
	{
		ucValue |= ~*(uint8_t erom*)uAddr;	// do not burn the bits are already low

		if (uAddr & 1)
			usValue = 0xFF00 | (uint16_t)ucValue;
		else
			usValue = 0x00FF | ((uint16_t)ucValue << 8);

		UnlockFlash(uAddr);
		*(uint16_t erom*)(uAddr & ~1ul) = usValue;
		LockFlash();
	}

	return *(uint8_t erom*)uAddr == ucValue;
}
/*
// This function writes a single byte value to the Flash location. It assumes
// the Page that corresponds to the location needs to be erased.
void EraseAndWriteFlash(uint32_t uAddr, uint8_t ucValue)
{
	uint8_t ucPageData[PAGE_SIZE];
	uint32_t uPageAddr = uAddr & -PAGE_SIZE;
	uint16_t erom* pusFlash;
	uint16_t* pusRam;

	// save the Flash page in RAM
	for (pusFlash = (uint16_t erom*)uPageAddr, pusRam = (uint16_t*)ucPageData; pusRam < (uint16_t*)&ucPageData[PAGE_SIZE]; )
		*pusRam++ = *pusFlash++;

	// Modify the byte location in RAM that corresponds to Flash saved page
	ucPageData[uAddr - uPageAddr] = ucValue;
				
	EraseFlash(uAddr);	// erase Flash Page

	UnlockFlash(uAddr);

	// Restore the page from RAM back to Flash
	for (pusRam = (uint16_t*)ucPageData, pusFlash = (uint16_t erom*)uPageAddr; pusRam < (uint16_t*)&ucPageData[PAGE_SIZE]; )
		*pusFlash++ = *pusRam++;

	LockFlash();
}
*/